	.h8300h
	.text
h8300h_rotate_shift:
	rotl.b r0l
	rotl.w r0
	rotl.l er0
	rotr.b r0l
	rotr.w r0
	rotr.l er0
	rotxl.b r0l
	rotxl.w r0
	rotxl.l er0
	rotxr.b r0l
	rotxr.w r0
	rotxr.l er0
	shal.b r0l
	shal.w r0
	shal.l er0
	shar.b r0l
	shar.w r0
	shar.l er0
	shll.b r0l
	shll.w r0
	shll.l er0
	shlr.b r0l
	shlr.w r0
	shlr.l er0
