/* Imitation sys/types.h. */

#ifndef __SYS_TYPES_H__
#define __SYS_TYPES_H__

#include <Types.h>

typedef short dev_t;
typedef short ino_t;
typedef unsigned short mode_t;
typedef unsigned short uid_t;
typedef unsigned short gid_t;
typedef long off_t;

#endif /* __SYS_TYPES_H__ */
