	dld.b	r4:s(r6),r8		; Register form
	dld.h	r4:s(r6),r8		; Register form
	dld	r4:s(r6),r8		; Register form
	dld.d	r4:s(r6),r8		; Register form
	dld.ub	r4:s(r6),r8		; Register form
	dld.uh	r4:s(r6),r8		; Register form
	dst.b	r4:s(r6),r8		; Register form
	dst.h	r4:s(r6),r8		; Register form
	dst	r4:s(r6),r8		; Register form
	dst.d	r4:s(r6),r8		; Register form
	ld.b	r4:s(r6),r8		; Register form
	ld.h	r4:s(r6),r8		; Register form
	ld	r4:s(r6),r8		; Register form
	ld.d	r4:s(r6),r8		; Register form
	ld.ub	r4:s(r6),r8		; Register form
	ld.uh	r4:s(r6),r8		; Register form
	st.b	r4:s(r6),r8		; Register form
	st.h	r4:s(r6),r8		; Register form
	st	r4:s(r6),r8		; Register form
	st.d	r4:s(r6),r8		; Register form
