
	.text
	.global arith_opcodes
arith_opcodes:
	add r5,r6
	add 5,r6
	addi 7,r5,r6
	divh r5,r6
	mulh r5,r6
	mulh 5,r6
	mulhi 7,r5,r6
	sar r5,r6
	sar 31,r6
	satadd r5,r6
	satadd 5,r6
	satsub r5,r6
	satsubi 7,r5,r6
	satsubr r5,r6
	shl r5,r6
	shl 31,r6
	shr r5,r6
	shr 31,r6
	sub r5,r6
	subr r5,r6
